####################################################
#                                                  #
#        2-day Network Analysis Workshop           #
#               updated January 2018               #
#                                                  #
#       All materials freely available at:         #
#    https://adelaisvoranu.com/Oldenburg2018       #
#                                                  #
#                                                  #
#                                                  #
####################################################


#######################################
### Part 1: Visualizing the network ###
#######################################

## Exercise 1. What do the nodes represent?
# Nodes represent characters and edges the number of times they are mentioned together. 

# Read data:
Data <- read.csv("stormofswords.csv")

View(Data)
## Exercise 1. What do the rows stand for and what do the columns stand for?
# Each row indicates an edge. 
# The first and second columns indicate the nodes an edge is connected to and 
# the third column indicates the strength of connection.
# This structure is known as an edgelist encoding a network,
# which can also be used as input for qgraph:

library("qgraph")
qgraph(Data, directed = FALSE)

## Exercise 2. Why did I had to use the directed argument? What does it do?
# An edgelist does not know if edges are directed (one-way) or undirected, 
# so we needed to spefify that.


Data2 <- read.csv("stormofswords_wmat.csv")
## This dataset represents the same network, but in a different way. 
## This representation is called a weights matrix.
View(Data2)

## Exercise 3. Use Data2 as input to qgraph(). Do you obtain the same network? 
## Do you still need to use the directed argument?

qgraph(Data2)

# When the weights matrix is symmetrical, it is automatically treated as undirected.
# Hence, we no longer needed the directed argument.

## Exercise 4. Recreate the plot above, changing the layout to a spring layout,
## the edge color to ???"darkblue"??? and the node size to 3. Look at the qgraph help page 
## to figure out the commands needed (?qgraph).
## Note: your computer might generate a different spring layout 
## (nodes placed on different locations).

qgraph(Data, directed = FALSE, layout = "spring", posCol = "darkblue", vsize = 3)
qgraph(Data2, layout = "spring", posCol = "darkblue", vsize = 3)

## Exercise 5. What values did qgraph set to cut and maximum? 
## Note: minimum is always set to 0 by default and not shown with details = TRUE 
## unless it differs from 0.

qgraph(Data, directed = FALSE, layout = "spring", posCol = "darkblue", vsize = 3, 
       details = TRUE)
qgraph(Data2, layout = "spring", posCol = "darkblue", vsize = 3, details = TRUE)

# qgraph set cut to 14 and maximum to 96.

## Exercise 6. Set the minimum argument to 1, 5 and 10 while using a spring layout. 
## How does the network change? Do the same using the threshold argument. 
## Can you explain why the layout remains the same using minimum but changes using threshold?

qgraph(Data, directed = FALSE, layout = "spring", posCol = "darkblue", vsize = 3, 
       details = TRUE, minimum=1)
qgraph(Data, directed = FALSE, layout = "spring", posCol = "darkblue", vsize = 3, 
       details = TRUE, minimum=5)
qgraph(Data, directed = FALSE, layout = "spring", posCol = "darkblue", vsize = 3, 
       details = TRUE, minimum=10)

# Minimum does not remove edges, hence the layout is the same. 
# The networks only visually differ.

## Exercise 7. Set the cut argument to 50, 10 and 1 while using a spring layout. 
## How does the network change? Now set the cut argument to 0. What happened?

qgraph(Data, directed = FALSE, layout = "spring", posCol = "darkblue", vsize = 3, 
       details = TRUE, cut=50)
qgraph(Data, directed = FALSE, layout = "spring", posCol = "darkblue", vsize = 3, 
       details = TRUE, cut=10)
qgraph(Data, directed = FALSE, layout = "spring", posCol = "darkblue", vsize = 3, 
       details = TRUE, cut=1)

# The higher cut is set, the less edges are highlighted. 
# Setting cut to 0 disabled the cutoff.

## Exercise 8. Set the maximum argument to 200. What happened? 
## Now set maximum back to its default and subsequently to 10. 
## Why doesn???t maximum = 10 change the network from its default value, 
## but maximum = 200 does?

qgraph(Data, directed = FALSE, layout = "spring", posCol = "darkblue", vsize = 3, 
       details = TRUE, maximum=200)
qgraph(Data, directed = FALSE, layout = "spring", posCol = "darkblue", vsize = 3, 
       details = TRUE)
qgraph(Data, directed = FALSE, layout = "spring", posCol = "darkblue", vsize = 3, 
       details = TRUE, maximum = 10)

# Setting maximum below the strongest edge in the network does nothing: 
# the edge weight width and saturation scale to is still the strongest edge weight 
# and not the maximum score then.

## Exercise 9. What would be a good setting for maximum when drawing networks based on
## (partial) correlations?

# A good setting would be 1.


#######################################
#### Part 2: Analyzing the network ####
#######################################

# Store the graph:
Graph <- qgraph(Data, directed = FALSE, layout = "spring")

# Obtain weights matrix:
WeightMat <- getWmat(Graph)

# colnames will give us the labels for each node
Labels <- colnames(WeightMat)

# Let's look for Jon Snow:
which(Labels == "Jon")
# [1] 32 
# Jon Snow is node 32

# And for Daenerys
which(Labels == "Daenerys")
## [1] 17
# Daenerys is node 17

# Let's have a look at centrality:
Centrality <- centrality(Graph, all.shortest.paths = TRUE)

# Node strength (also called degree) sums the connected edge weights to a node. 
# The node strength of Jon and Deanerys are:

Centrality$OutDegree[c(17, 32)]
## Daenerys      Jon
##      232      442

# Closeness computes how ???close??? two nodes are together in the network:
Centrality$Closeness[c(17, 32)]
##   Daenerys        Jon
## 0.02612782 0.04305410

# Finally, Betweenness computes how often one node is featured in the most efficient 
# (shortest) paths between other nodes:

Centrality$Betweenness[c(17, 32)]
## Daenerys      Jon
##     2200     2838

# Based on these results, Jon seems more important in this book.

## Exercise 10.
# Compare the centrality of the character named ???Tyrion??? to those of Jon and Daenerys.

# Tyrion is node:
which(Labels == "Tyrion")
# [1] 64

# Centrality:
Centrality$OutDegree[64]
# Tyrion
#    551

Centrality$Closeness[64]
#     Tyrion
# 0.05291881

Centrality$Betweenness[64]
# Tyrion
#   3938

# Tyrion is more central!

# We can also look at how ???close??? Jon and Daenerys are:
Centrality$ShortestPathLengths[17,32]
## [1] 0.3969188  

# The following characters connect Jon and Daenerys:
Labels[Centrality$ShortestPaths[[17,32]][[1]]]
## [1] "Daenerys" "Robert"   "Jaime"    "Robb"     "Jon"

## Exercise 11. Via which characters is Tyrion connected to Jon and Daenerys?
# Tyrion to Jon:
Labels[Centrality$ShortestPaths[[64,32]][[1]]]
# [1] "Tyrion" "Sansa"  "Robb"   "Jon"

# Tyrion to Daenerys:
Labels[Centrality$ShortestPaths[[64,17]][[1]]]
# [1] "Tyrion"   "Cersei"   "Robert"   "Daenerys"


# We can plot the centrality indices using the centralityPlot function:
centralityPlot(Graph)

## Exercise 12. What character would you conclude is the most central? 
# Tyrion. Also central are Sansa and Robb.


# Many network analysts prefer to analyze an unweighted network rather than a weighted 
# network. We can strip the weights by setting all edge weights to 1 or by not using 
# the weights in the edgelist:
Graph_unweighted <- qgraph(Data[,1:2], directed=FALSE)
Graph_unweighted <- qgraph(Data[,1:2], directed=FALSE, repulsion = 0.7)

##Exercise 13. What happened when I set the repulsion argument lower than its default of 1? 
# Nodes are drawn further away from each-other.

## Exercise 14. Recreate the centrality plot using the unweighted network. 
## Does your conclusion on the most central character change?
centralityPlot(Graph_unweighted)

# Tyrion is now more clearly the most central character.

## Exercise 15. Does the unweighted network feature a small world? 
smallworldIndex(Graph_unweighted)$index
# [1] 4.921709

# Yes, because the index is above 3
